@@ -1,4 +1,4 @@
-// Copyright (C) 2002-2009 Nikolaus Gebhardt
+// Copyright (C) 2002-2012 Nikolaus Gebhardt
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -14,6 +14,7 @@
 #include "triangle3d.h"
 #include "position2d.h"
 #include "rect.h"
+#include "dimension2d.h"
 #include "matrix4.h"
 #include "quaternion.h"
 #include "plane3d.h"
@@ -24,6 +25,7 @@
 #include "irrArray.h"
 #include "IXMLReader.h"
 #include "EAttributes.h"
+#include "path.h"
 
 namespace irr
 {
@@ -67,7 +69,7 @@ public:
 	virtual bool existsAttribute(const c8* attributeName) = 0;
 
 	//! Returns attribute index from name, -1 if not found
-	virtual s32 findAttribute(const c8* attributeName) = 0;
+	virtual s32 findAttribute(const c8* attributeName) const =0;
 
 	//! Removes all attributes
 	virtual void clear() = 0;
@@ -102,11 +104,11 @@ public:
 	//! Gets an attribute as integer value
 	//! \param attributeName: Name of the attribute to get.
 	//! \return Returns value of the attribute previously set by setAttribute()
-	virtual s32 getAttributeAsInt(const c8* attributeName) = 0;
+	virtual s32 getAttributeAsInt(const c8* attributeName) const =0;
 
 	//! Gets an attribute as integer value
 	//! \param index: Index value, must be between 0 and getAttributeCount()-1.
-	virtual s32 getAttributeAsInt(s32 index) = 0;
+	virtual s32 getAttributeAsInt(s32 index) const =0;
 
 	//! Sets an attribute as integer value
 	virtual void setAttribute(s32 index, s32 value) = 0;
@@ -411,6 +413,30 @@ public:
 	//! Sets an attribute as vector
 	virtual void setAttribute(s32 index, core::vector3df v) = 0;
 
+	/*
+
+		Vector2d Attribute
+
+	*/
+
+	//! Adds an attribute as 2d vector
+	virtual void addVector2d(const c8* attributeName, core::vector2df value) = 0;
+
+	//! Sets a attribute as 2d vector
+	virtual void setAttribute(const c8* attributeName, core::vector2df v) = 0;
+
+	//! Gets an attribute as vector
+	//! \param attributeName: Name of the attribute to get.
+	//! \return Returns value of the attribute previously set by setAttribute()
+	virtual core::vector2df getAttributeAsVector2d(const c8* attributeName) = 0;
+
+	//! Gets an attribute as position
+	//! \param index: Index value, must be between 0 and getAttributeCount()-1.
+	virtual core::vector2df getAttributeAsVector2d(s32 index) = 0;
+
+	//! Sets an attribute as 2d vector
+	virtual void setAttribute(s32 index, core::vector2df v) = 0;
+
 	/*
 
 		Position2d Attribute
@@ -460,6 +486,31 @@ public:
 	virtual void setAttribute(s32 index, core::rect<s32> v) = 0;
 
 
+	/*
+
+		Dimension2d Attribute
+
+	*/
+
+	//! Adds an attribute as dimension2d
+	virtual void addDimension2d(const c8* attributeName, core::dimension2d<u32> value) = 0;
+
+	//! Sets an attribute as dimension2d
+	virtual void setAttribute(const c8* attributeName, core::dimension2d<u32> v) = 0;
+
+	//! Gets an attribute as dimension2d
+	//! \param attributeName: Name of the attribute to get.
+	//! \return Returns value of the attribute previously set by setAttribute()
+	virtual core::dimension2d<u32> getAttributeAsDimension2d(const c8* attributeName) = 0;
+
+	//! Gets an attribute as dimension2d
+	//! \param index: Index value, must be between 0 and getAttributeCount()-1.
+	virtual core::dimension2d<u32> getAttributeAsDimension2d(s32 index) = 0;
+
+	//! Sets an attribute as dimension2d
+	virtual void setAttribute(s32 index, core::dimension2d<u32> v) = 0;
+
+
 	/*
 		matrix attribute
 	*/
@@ -636,10 +687,10 @@ public:
 	*/
 
 	//! Adds an attribute as texture reference
-	virtual void addTexture(const c8* attributeName, video::ITexture* texture) = 0;
+	virtual void addTexture(const c8* attributeName, video::ITexture* texture, const io::path& filename = "") = 0;
 
 	//! Sets an attribute as texture reference
-	virtual void setAttribute(const c8* attributeName, video::ITexture* texture ) = 0;
+	virtual void setAttribute(const c8* attributeName, video::ITexture* texture, const io::path& filename = "") = 0;
 
 	//! Gets an attribute as texture reference
 	//! \param attributeName: Name of the attribute to get.
@@ -650,7 +701,7 @@ public:
 	virtual video::ITexture* getAttributeAsTexture(s32 index) = 0;
 
 	//! Sets an attribute as texture reference
-	virtual void setAttribute(s32 index, video::ITexture* texture) = 0;
+	virtual void setAttribute(s32 index, video::ITexture* texture, const io::path& filename = "") = 0;
 
 
 	/*
