@@ -1,4 +1,4 @@
-// Copyright (C) 2009-2010 Gaz Davidson
+// Copyright (C) 2009-2012 Gaz Davidson
 // This file is part of the "Irrlicht Engine".
 // For conditions of distribution and use, see copyright notice in irrlicht.h
 
@@ -23,8 +23,8 @@ namespace scene
 #define PLY_INPUT_BUFFER_SIZE 51200 // file is loaded in 50k chunks
 
 // constructor
-CPLYMeshFileLoader::CPLYMeshFileLoader()
-: File(0), Buffer(0)
+CPLYMeshFileLoader::CPLYMeshFileLoader(scene::ISceneManager* smgr)
+: SceneManager(smgr), File(0), Buffer(0)
 {
 }
 
@@ -235,6 +235,7 @@ IAnimatedMesh* CPLYMeshFileLoader::createMesh(io::IReadFile* file)
 			mb->getIndexBuffer().reallocate(vertCount);
 			mb->setHardwareMappingHint(EHM_STATIC);
 
+			bool hasNormals=true;
 			// loop through each of the elements
 			for (u32 i=0; i<ElementList.size(); ++i)
 			{
@@ -243,7 +244,7 @@ IAnimatedMesh* CPLYMeshFileLoader::createMesh(io::IReadFile* file)
 				{
 					// loop through vertex properties
 					for (u32 j=0; j < ElementList[i]->Count; ++j)
-						readVertex(*ElementList[i], mb);
+						hasNormals &= readVertex(*ElementList[i], mb);
 				}
 				else if (ElementList[i]->Name == "face")
 				{
@@ -259,6 +260,8 @@ IAnimatedMesh* CPLYMeshFileLoader::createMesh(io::IReadFile* file)
 				}
 			}
 			mb->recalculateBoundingBox();
+			if (!hasNormals)
+				SceneManager->getMeshManipulator()->recalculateNormals(mb);
 			SMesh* m = new SMesh();
 			m->addMeshBuffer(mb);
 			m->recalculateBoundingBox();
@@ -295,6 +298,7 @@ bool CPLYMeshFileLoader::readVertex(const SPLYElement &Element, scene::CDynamicM
 	vert.Normal.Y = 1.0f;
 	vert.Normal.Z = 0.0f;
 
+	bool result=false;
 	for (u32 i=0; i < Element.Properties.size(); ++i)
 	{
 		E_PLY_PROPERTY_TYPE t = Element.Properties[i].Type;
@@ -306,11 +310,20 @@ bool CPLYMeshFileLoader::readVertex(const SPLYElement &Element, scene::CDynamicM
 		else if (Element.Properties[i].Name == "z")
 			vert.Pos.Y = getFloat(t);
 		else if (Element.Properties[i].Name == "nx")
+		{
 			vert.Normal.X = getFloat(t);
+			result=true;
+		}
 		else if (Element.Properties[i].Name == "ny")
+		{
 			vert.Normal.Z = getFloat(t);
+			result=true;
+		}
 		else if (Element.Properties[i].Name == "nz")
+		{
 			vert.Normal.Y = getFloat(t);
+			result=true;
+		}
 		else if (Element.Properties[i].Name == "u")
 			vert.TCoords.X = getFloat(t);
 		else if (Element.Properties[i].Name == "v")
@@ -341,7 +354,7 @@ bool CPLYMeshFileLoader::readVertex(const SPLYElement &Element, scene::CDynamicM
 
 	mb->getVertexBuffer().push_back(vert);
 
-	return true;
+	return result;
 }
 
 
@@ -456,7 +469,7 @@ void CPLYMeshFileLoader::fillBuffer()
 	if (EndOfFile)
 		return;
 
-	u32 length = EndPointer - StartPointer;
+	u32 length = (u32)(EndPointer - StartPointer);
 	if (length && StartPointer != Buffer)
 	{
 		// copy the remaining data to the start of the buffer
@@ -634,7 +647,7 @@ c8* CPLYMeshFileLoader::getNextWord()
 		++pos;
 	}
 	--pos;
-	WordLength = pos-StartPointer;
+	WordLength = (s32)(pos-StartPointer);
 	// return pointer to the start of the word
 	return StartPointer;
 }
